@@ -50,7 +50,8 @@ ifs_3_should_warn: {
         conditionals : true,
         dead_code    : true,
         evaluate     : true,
-        booleans     : true
+        booleans     : true,
+        side_effects : true,
     };
     input: {
         var x, y;
@@ -135,16 +136,28 @@ ifs_6: {
         comparisons: true
     };
     input: {
-        var x;
+        var x, y;
         if (!foo && !bar && !baz && !boo) {
             x = 10;
         } else {
             x = 20;
         }
+        if (y) {
+            x[foo] = 10;
+        } else {
+            x[foo] = 20;
+        }
+        if (foo) {
+            x[bar] = 10;
+        } else {
+            x[bar] = 20;
+        }
     }
     expect: {
-        var x;
+        var x, y;
         x = foo || bar || baz || boo ? 20 : 10;
+        x[foo] = y ? 10 : 20;
+        foo ? x[bar] = 10 : x[bar] = 20;
     }
 }
 
@@ -159,10 +172,16 @@ cond_1: {
         } else {
             do_something(y);
         }
+        if (some_condition()) {
+            side_effects(x);
+        } else {
+            side_effects(y);
+        }
     }
     expect: {
         var do_something;
         do_something(some_condition() ? x : y);
+        some_condition() ? side_effects(x) : side_effects(y);
     }
 }
 
@@ -213,10 +232,16 @@ cond_4: {
         } else {
             do_something();
         }
+        if (some_condition()) {
+            side_effects();
+        } else {
+            side_effects();
+        }
     }
     expect: {
         var do_something;
         some_condition(), do_something();
+        some_condition(), side_effects();
     }
 }
 
@@ -250,7 +275,8 @@ cond_5: {
 cond_7: {
     options = {
         conditionals: true,
-        evaluate    : true
+        evaluate    : true,
+        side_effects: true,
     };
     input: {
         var x, y, z, a, b;
@@ -635,166 +661,6 @@ ternary_boolean_alternative: {
     }
 }
 
-conditional_and: {
-    options = {
-        conditionals: true,
-        evaluate    : true
-    };
-    input: {
-        var a;
-        // compress these
-
-        a = true     && condition;
-        a = 1        && console.log("a");
-        a = 2 * 3    && 2 * condition;
-        a = 5 == 5   && condition + 3;
-        a = "string" && 4 - condition;
-        a = 5 + ""   && condition / 5;
-        a = -4.5     && 6 << condition;
-        a = 6        && 7;
-
-        a = false     && condition;
-        a = NaN       && console.log("b");
-        a = 0         && console.log("c");
-        a = undefined && 2 * condition;
-        a = null      && condition + 3;
-        a = 2 * 3 - 6 && 4 - condition;
-        a = 10 == 7   && condition / 5;
-        a = !"string" && 6 % condition;
-        a = 0         && 7;
-
-        // don't compress these
-
-        a = condition        && true;
-        a = console.log("a") && 2;
-        a = 4 - condition    && "string";
-        a = 6 << condition   && -4.5;
-
-        a = condition        && false;
-        a = console.log("b") && NaN;
-        a = console.log("c") && 0;
-        a = 2 * condition    && undefined;
-        a = condition + 3    && null;
-
-    }
-    expect: {
-        var a;
-
-        a = condition;
-        a = console.log("a");
-        a = 2 * condition;
-        a = condition + 3;
-        a = 4 - condition;
-        a = condition / 5;
-        a = 6 << condition;
-        a = 7;
-
-        a = false;
-        a = NaN;
-        a = 0;
-        a = void 0;
-        a = null;
-        a = 0;
-        a = false;
-        a = false;
-        a = 0;
-
-        a = condition        && true;
-        a = console.log("a") && 2;
-        a = 4 - condition    && "string";
-        a = 6 << condition   && -4.5;
-
-        a = condition        && false;
-        a = console.log("b") && NaN;
-        a = console.log("c") && 0;
-        a = 2 * condition    && void 0;
-        a = condition + 3    && null;
-    }
-}
-
-conditional_or: {
-    options = {
-        conditionals: true,
-        evaluate    : true
-    };
-    input: {
-        var a;
-        // compress these
-
-        a = true     || condition;
-        a = 1        || console.log("a");
-        a = 2 * 3    || 2 * condition;
-        a = 5 == 5   || condition + 3;
-        a = "string" || 4 - condition;
-        a = 5 + ""   || condition / 5;
-        a = -4.5     || 6 << condition;
-        a = 6        || 7;
-
-        a = false     || condition;
-        a = 0         || console.log("b");
-        a = NaN       || console.log("c");
-        a = undefined || 2 * condition;
-        a = null      || condition + 3;
-        a = 2 * 3 - 6 || 4 - condition;
-        a = 10 == 7   || condition / 5;
-        a = !"string" || 6 % condition;
-        a = null      || 7;
-
-        a = console.log(undefined && condition || null);
-        a = console.log(undefined || condition && null);
-
-        // don't compress these
-
-        a = condition        || true;
-        a = console.log("a") || 2;
-        a = 4 - condition    || "string";
-        a = 6 << condition   || -4.5;
-
-        a = condition        || false;
-        a = console.log("b") || NaN;
-        a = console.log("c") || 0;
-        a = 2 * condition    || undefined;
-        a = condition + 3    || null;
-
-    }
-    expect: {
-        var a;
-
-        a = true;
-        a = 1;
-        a = 6;
-        a = true;
-        a = "string";
-        a = "5";
-        a = -4.5;
-        a = 6;
-
-        a = condition;
-        a = console.log("b");
-        a = console.log("c");
-        a = 2 * condition;
-        a = condition + 3;
-        a = 4 - condition;
-        a = condition / 5;
-        a = 6 % condition;
-        a = 7;
-
-        a = console.log(null);
-        a = console.log(condition && null);
-
-        a = condition        || true;
-        a = console.log("a") || 2;
-        a = 4 - condition    || "string";
-        a = 6 << condition   || -4.5;
-
-        a = condition        || false;
-        a = console.log("b") || NaN;
-        a = console.log("c") || 0;
-        a = 2 * condition    || void 0;
-        a = condition + 3    || null;
-    }
-}
-
 trivial_boolean_ternary_expressions : {
     options = {
         conditionals: true,
@@ -874,6 +740,7 @@ issue_1154: {
         conditionals: true,
         evaluate    : true,
         booleans    : true,
+        side_effects: true,
     };
     input: {
         function f1(x) { return x ? -1 : -1; }
@@ -902,7 +769,249 @@ issue_1154: {
         function g2() { return g(), 2; }
         function g3() { return g(), -4; }
         function g4() { return g(), !1; }
-        function g5() { return g(), void 0; }
+        function g5() { return void g(); }
         function g6() { return g(), "number"; }
     }
 }
+
+no_evaluate: {
+    options = {
+        conditionals: true,
+        evaluate    : false,
+        side_effects: true,
+    }
+    input: {
+        function f(b) {
+            a = b ? !0 : !0;
+            a = b ? ~1 : ~1;
+            a = b ? -2 : -2;
+            a = b ? +3 : +3;
+        }
+    }
+    expect: {
+        function f(b) {
+            a = !0;
+            a = ~1;
+            a = -2;
+            a = +3;
+        }
+    }
+}
+
+equality_conditionals_false: {
+    options = {
+        conditionals: false,
+        sequences: true,
+    }
+    input: {
+        function f(a, b, c) {
+            console.log(
+                a == (b ? a : a),
+                a == (b ? a : c),
+                a != (b ? a : a),
+                a != (b ? a : c),
+                a === (b ? a : a),
+                a === (b ? a : c),
+                a !== (b ? a : a),
+                a !== (b ? a : c)
+            );
+        }
+        f(0, 0, 0);
+        f(0, true, 0);
+        f(1, 2, 3);
+        f(1, null, 3);
+        f(NaN);
+        f(NaN, "foo");
+    }
+    expect: {
+        function f(a, b, c) {
+            console.log(
+                a == (b ? a : a),
+                a == (b ? a : c),
+                a != (b ? a : a),
+                a != (b ? a : c),
+                a === (b ? a : a),
+                a === (b ? a : c),
+                a !== (b ? a : a),
+                a !== (b ? a : c)
+            );
+        }
+        f(0, 0, 0),
+        f(0, true, 0),
+        f(1, 2, 3),
+        f(1, null, 3),
+        f(NaN),
+        f(NaN, "foo");
+    }
+    expect_stdout: true
+}
+
+equality_conditionals_true: {
+    options = {
+        conditionals: true,
+        sequences: true,
+    }
+    input: {
+        function f(a, b, c) {
+            console.log(
+                a == (b ? a : a),
+                a == (b ? a : c),
+                a != (b ? a : a),
+                a != (b ? a : c),
+                a === (b ? a : a),
+                a === (b ? a : c),
+                a !== (b ? a : a),
+                a !== (b ? a : c)
+            );
+        }
+        f(0, 0, 0);
+        f(0, true, 0);
+        f(1, 2, 3);
+        f(1, null, 3);
+        f(NaN);
+        f(NaN, "foo");
+    }
+    expect: {
+        function f(a, b, c) {
+            console.log(
+                (b, a == a),
+                a == (b ? a : c),
+                (b, a != a),
+                a != (b ? a : c),
+                (b, a === a),
+                a === (b ? a : c),
+                (b, a !== a),
+                a !== (b ? a : c)
+            );
+        }
+        f(0, 0, 0),
+        f(0, true, 0),
+        f(1, 2, 3),
+        f(1, null, 3),
+        f(NaN),
+        f(NaN, "foo");
+    }
+    expect_stdout: true
+}
+
+issue_1645_1: {
+    options = {
+        conditionals: true,
+    }
+    input: {
+        var a = 100, b = 10;
+        (b = a) ? a++ + (b += a) ? b += a : b += a : b ^= a;
+        console.log(a, b);
+    }
+    expect: {
+        var a = 100, b = 10;
+        (b = a) ? (a++ + (b += a), b += a) : b ^= a;
+        console.log(a,b);
+    }
+    expect_stdout: true
+}
+
+issue_1645_2: {
+    options = {
+        conditionals: true,
+    }
+    input: {
+        var a = 0;
+        function f() {
+            return a++;
+        }
+        f() ? a += 2 : a += 4;
+        console.log(a);
+    }
+    expect: {
+        var a = 0;
+        function f(){
+            return a++;
+        }
+        f() ? a += 2 : a += 4;
+        console.log(a);
+    }
+    expect_stdout: true
+}
+
+condition_symbol_matches_consequent: {
+    options = {
+        conditionals: true,
+    }
+    input: {
+        function foo(x, y) {
+            return x ? x : y;
+        }
+        function bar() {
+            return g ? g : h;
+        }
+        var g = 4;
+        var h = 5;
+        console.log(foo(3, null), foo(0, 7), foo(true, false), bar());
+    }
+    expect: {
+        function foo(x, y) {
+            return x || y;
+        }
+        function bar() {
+            return g || h;
+        }
+        var g = 4;
+        var h = 5;
+        console.log(foo(3, null), foo(0, 7), foo(true, false), bar());
+    }
+    expect_stdout: "3 7 true 4"
+}
+
+delete_conditional_1: {
+    options = {
+        booleans: true,
+        conditionals: true,
+        evaluate: true,
+        side_effects: true,
+    }
+    input: {
+        console.log(delete (1 ? undefined : x));
+        console.log(delete (1 ? void 0 : x));
+        console.log(delete (1 ? Infinity : x));
+        console.log(delete (1 ? 1 / 0 : x));
+        console.log(delete (1 ? NaN : x));
+        console.log(delete (1 ? 0 / 0 : x));
+    }
+    expect: {
+        console.log((void 0, !0));
+        console.log((void 0, !0));
+        console.log((1 / 0, !0));
+        console.log((1 / 0, !0));
+        console.log((NaN, !0));
+        console.log((NaN, !0));
+    }
+    expect_stdout: true
+}
+
+delete_conditional_2: {
+    options = {
+        booleans: true,
+        conditionals: true,
+        evaluate: true,
+        keep_infinity: true,
+        side_effects: true,
+    }
+    input: {
+        console.log(delete (0 ? x : undefined));
+        console.log(delete (0 ? x : void 0));
+        console.log(delete (0 ? x : Infinity));
+        console.log(delete (0 ? x : 1 / 0));
+        console.log(delete (0 ? x : NaN));
+        console.log(delete (0 ? x : 0 / 0));
+    }
+    expect: {
+        console.log((void 0, !0));
+        console.log((void 0, !0));
+        console.log((Infinity, !0));
+        console.log((1 / 0, !0));
+        console.log((NaN, !0));
+        console.log((NaN, !0));
+    }
+    expect_stdout: true
+}
